@@ -8,13 +8,14 @@ from pathlib import Path
 import numpy as np
 
 
-def min_fast(a, b):
+def fast_min(a, b):
     '''
     1.5 times faster than row min(a, b)
     '''
-    return  a if a < b else b
+    return a if a < b else b
 
-def max_fast(a, b):
+
+def fast_max(a, b):
     return a if a > b else b
 
 
@@ -35,7 +36,7 @@ def split_matrix(mat: np.ndarray):
     """
     splits wide pop matrix to variables and scores
     """
-    return mat[:,:-1], mat[:, -1]
+    return mat[:, :-1], mat[:, -1]
 
 
 def union_to_matrix(variables_2D: np.ndarray, scores_1D: np.ndarray):
